# SPDX-License-Identifier: Apache-2.0.
# Copyright (c) 2024 - 2025 Waldiez and contributors.
"""Waldiez model model."""

import os
from typing import Any, Dict, List, Optional

from pydantic import Field
from typing_extensions import Annotated, Literal

from ..common import WaldiezBase, now
from .model_data import WaldiezModelAPIType, WaldiezModelData

DEFAULT_BASE_URLS: Dict[WaldiezModelAPIType, str] = {
    "google": "https://generativelanguage.googleapis.com/v1beta",
    "anthropic": "https://api.anthropic.com/v1",
    "mistral": "https://api.mistral.ai/v1",
    "groq": "https://api.groq.com/openai/v1",
    "together": "https://api.together.xyz/v1",
    "nim": "https://integrate.api.nvidia.com/v1",
}


class WaldiezModel(WaldiezBase):
    """Waldiez Model class.

    Attributes
    ----------
    id : str
        The ID of the model.
    name : str
        The name of the model.
    description : str
        The description of the model.
    tags : List[str]
        The tags of the model.
    requirements : List[str]
        The requirements of the model.
    created_at : str
        The date and time when the model was created.
    updated_at : str
        The date and time when the model was last updated.
    data : WaldiezModelData
        The data of the model.
        See `waldiez.models.model.WaldiezModelData` for more information.
    """

    id: Annotated[
        str, Field(..., title="ID", description="The ID of the model.")
    ]
    type: Annotated[
        Literal["model"],
        Field(
            default="model",
            title="Type",
            description="The type of the 'node' in a graph.",
        ),
    ]
    name: Annotated[
        str, Field(..., title="Name", description="The name of the model.")
    ]
    description: Annotated[
        str,
        Field(
            ...,
            title="Description",
            description="The description of the model.",
        ),
    ]
    tags: Annotated[
        List[str],
        Field(
            default_factory=list,
            title="Tags",
            description="The tags of the model.",
        ),
    ]
    requirements: Annotated[
        List[str],
        Field(
            default_factory=list,
            title="Requirements",
            description="The requirements of the model.",
        ),
    ]
    created_at: Annotated[
        str,
        Field(
            default_factory=now,
            title="Create At",
            description="The date and time when the model was created.",
        ),
    ]
    updated_at: Annotated[
        str,
        Field(
            default_factory=now,
            title="Updated At",
            description="The date and time when the model was last updated.",
        ),
    ]
    data: Annotated[
        WaldiezModelData,
        Field(..., title="Data", description="The data of the model."),
    ]

    @property
    def api_key_env_key(self) -> str:
        """Get the model's api key environment key to check.

        - openai: 'OPENAI_API_KEY',
        - azure: 'AZURE_API_KEY',
        - google: 'GOOGLE_GEMINI_API_KEY',
        - anthropic: 'ANTHROPIC_API_KEY',
        - mistral: 'MISTRAL_API_KEY',
        - groq: 'GROQ_API_KEY',
        - together: 'TOGETHER_API_KEY',
        - nim: 'NIM_API_KEY',
        - other: 'OPENAI_API_KEY'
        """
        env_key = "OPENAI_API_KEY"
        if self.data.api_type == "google":
            env_key = "GOOGLE_GEMINI_API_KEY"
        elif self.data.api_type not in ["openai", "other"]:
            env_key = f"{self.data.api_type.upper()}_API_KEY"
        return env_key

    @property
    def api_key(self) -> str:
        """Get the model's api key.

        Either from the model's data or from the environment variables:

            - openai: 'OPENAI_API_KEY',
            - azure: 'AZURE_API_KEY',
            - google: 'GOOGLE_GEMINI_API_KEY',
            - anthropic: 'ANTHROPIC_API_KEY',
            - mistral: 'MISTRAL_API_KEY',
            - groq: 'GROQ_API_KEY',
            - together: 'TOGETHER_API_KEY',
            - nim: 'NIM_API_KEY',
            - other: 'OPENAI_API_KEY'
        """
        if self.data.api_key and self.data.api_key != "REPLACE_ME":
            return self.data.api_key
        env_key = self.api_key_env_key
        api_key = os.environ.get(
            env_key, getattr(self.data, "api_key", "REPLACE_ME")
        )
        return api_key or "REPLACE_ME"

    @property
    def price(self) -> Optional[List[float]]:
        """Get the model's price."""
        if self.data.price is None:
            return None
        if isinstance(
            self.data.price.prompt_price_per_1k, float
        ) and isinstance(self.data.price.completion_token_price_per_1k, float):
            return [
                self.data.price.prompt_price_per_1k,
                self.data.price.completion_token_price_per_1k,
            ]
        return None

    def get_llm_config(self, skip_price: bool = False) -> Dict[str, Any]:
        """Get the model's llm config.

        Parameters
        ----------
        skip_price : bool, optional
            Whether to skip the price, by default False

        Returns
        -------
        Dict[str, Any]
            The model's llm config dictionary.
        """
        _llm_config: Dict[str, Any] = {}
        _llm_config["model"] = self.name
        for attr, atr_type in [
            ("base_url", str),
            ("max_tokens", int),
            ("temperature", float),
            ("top_p", float),
            ("api_version", str),
            ("default_headers", dict),
        ]:
            value = getattr(self.data, attr)
            if value and isinstance(value, atr_type):
                _llm_config[attr] = value
        if self.data.api_type not in ["nim", "other"]:
            _llm_config["api_type"] = self.data.api_type
        other_attrs = ["api_key"] if skip_price else ["api_key", "price"]
        for attr in other_attrs:
            value = getattr(self, attr)
            if value:
                _llm_config[attr] = value
        return set_default_base_url(_llm_config, self.data.api_type)


def set_default_base_url(
    llm_config: Dict[str, Any], api_type: WaldiezModelAPIType
) -> Dict[str, Any]:
    """Set the default base url if not provided.

    Parameters
    ----------
    llm_config : Dict[str, Any]
        The llm config dictionary.
    api_type : str
        The api type.

    Returns
    -------
    Dict[str, Any]
        The llm config dictionary with the default base url set.
    """
    if api_type in ("openai", "other", "azure"):
        return llm_config
    if "base_url" not in llm_config or not llm_config["base_url"]:
        dict_copy = llm_config.copy()
        dict_copy["base_url"] = DEFAULT_BASE_URLS.get(api_type, "")
        return dict_copy
    return llm_config



# SPDX-License-Identifier: Apache-2.0.
# Copyright (c) 2024 - 2025 Waldiez and contributors.
"""Waldiez package."""

import logging
import warnings

from ._version import __version__
from .conflict_checker import check_conflicts
from .exporter import WaldiezExporter
from .models import Waldiez
from .runner import WaldiezRunner

warnings.filterwarnings(
    "ignore", module="flaml", message="^.*flaml.automl is not available.*$"
)


# pylint: disable=too-few-public-methods
class FlamlFilter(logging.Filter):
    """Filter out flaml.automl is not available message."""

    def filter(self, record: logging.LogRecord) -> bool:
        """Filter out flaml.automl is not available message.

        this is just annoying:

        ```
        flaml.automl is not available.
        Please install flaml[automl] to enable AutoML functionalities.
        ```

        Parameters
        ----------
        record : logging.LogRecord
            Log record to filter.

        Returns
        -------
        bool
            Whether to filter out the log record.
        """
        return "flaml.automl is not available" not in record.getMessage()


# flag to check if ag2 and autogen-agentchat
# are installed at the same time
__WALDIEZ_CHECKED_FOR_CONFLICTS = False
# flag to handle flaml logging
# suppress the annoying message about flaml.automl
__WALDIEZ_HANDLED_FLAML_LOGGING = False


def _check_conflicts() -> None:
    """Check for conflicts once."""
    # pylint: disable=global-statement
    global __WALDIEZ_CHECKED_FOR_CONFLICTS
    if __WALDIEZ_CHECKED_FOR_CONFLICTS is False:
        check_conflicts()
        __WALDIEZ_CHECKED_FOR_CONFLICTS = True


def _handle_flaml_logging() -> None:
    """Handle flaml logging once."""
    # pylint: disable=global-statement
    global __WALDIEZ_HANDLED_FLAML_LOGGING
    if __WALDIEZ_HANDLED_FLAML_LOGGING is False:
        __WALDIEZ_HANDLED_FLAML_LOGGING = True
        flam_logger = logging.getLogger("flaml")
        flam_logger.addFilter(FlamlFilter())


_check_conflicts()
_handle_flaml_logging()

__all__ = [
    "Waldiez",
    "WaldiezExporter",
    "WaldiezRunner",
    "__version__",
]



# SPDX-License-Identifier: Apache-2.0.
# Copyright (c) 2024 - 2025 Waldiez and contributors.
"""Export group manager and group chat to string."""

from typing import Callable, Dict, List, Optional, Tuple

from waldiez.models import WaldiezAgent, WaldiezGroupManager


def get_group_manager_extras(
    agent: WaldiezAgent,
    group_chat_members: List[WaldiezAgent],
    agent_names: Dict[str, str],
    serializer: Callable[..., str],
) -> Tuple[str, str]:
    """Get the group manager extra string and custom selection method if any.

    Parameters
    ----------
    agent : WaldiezAgent
        The agent.
    group_chat_members : List[WaldiezAgent]
        The group members.
    agent_names : Dict[str, str]
        The agent names.
    serializer : Callable[..., str]
        The serializer function.

    Returns
    -------
    Tuple[str, str]
        The content before the agent string and the group chat argument.
    """

    group_chat_arg = ""
    before_agent_string = ""
    custom_speaker_selection: Optional[str] = None
    if agent.agent_type == "manager" and isinstance(agent, WaldiezGroupManager):
        group_chat_string, group_chat_name, custom_speaker_selection = (
            _get_group_manager_extras(
                agent=agent,
                group_members=group_chat_members,
                agent_names=agent_names,
                serializer=serializer,
            )
        )
        if group_chat_name:
            group_chat_arg = "\n" + f"    groupchat={group_chat_name},"
        if custom_speaker_selection:
            before_agent_string += f"{custom_speaker_selection}" + "\n"
        if group_chat_string:
            before_agent_string += group_chat_string
    return before_agent_string, group_chat_arg


def _get_group_manager_extras(
    agent: WaldiezGroupManager,
    group_members: List[WaldiezAgent],
    agent_names: Dict[str, str],
    serializer: Callable[..., str],
) -> Tuple[str, str, Optional[str]]:
    """Get the group manager extra string and custom selection method if any.

    Parameters
    ----------
    agent : WaldiezGroupManager
        The agent.
    group_members : List[WaldiezAgent]
        The group members.
    agent_names : Dict[str, str]
        The agent names.
    serializer : Callable[..., str]
        The serializer function.

    Returns
    -------
    str
        The group chat definition string.
    str
        The group chat name.
    Optional[str]
        The custom selection method name and content if any.
    """
    agent_name = agent_names[agent.id]
    group_chat_name = f"{agent_name}_group_chat"
    group_members_str = ", ".join(
        agent_names[member.id] for member in group_members
    )
    group_chat_string = "\n" + f"{group_chat_name} = GroupChat(" + "\n"
    group_chat_string += f"    agents=[{group_members_str}]," + "\n"
    group_chat_string += (
        f"    enable_clear_history={agent.data.enable_clear_history}," + "\n"
    )
    group_chat_string += (
        f"    send_introductions={agent.data.send_introductions}," + "\n"
    )
    group_chat_string += "    messages=[]," + "\n"
    if agent.data.max_round is not None:
        group_chat_string += f"    max_round={agent.data.max_round}," + "\n"
    if agent.data.admin_name:
        group_chat_string += f'    admin_name="{agent.data.admin_name}",' + "\n"
    extra_group_chat_string, custom_selection_method = (
        _get_group_chat_speakers_string(agent, agent_names, serializer)
    )
    group_chat_string += extra_group_chat_string
    group_chat_string += ")\n\n"
    return group_chat_string, group_chat_name, custom_selection_method


def _get_group_chat_speakers_string(
    agent: WaldiezGroupManager,
    agent_names: Dict[str, str],
    serializer: Callable[..., str],
) -> Tuple[str, Optional[str]]:
    """Get the group chat speakers string.

    Parameters
    ----------
    agent : WaldiezGroupManager
        The agent.
    agent_names : Dict[str, str]
        The agent names.
    serializer : Callable[..., str]
        The serializer function.

    Returns
    -------
    str
        The group chat speakers string.
    Optional[str]
        The custom custom for speaker selection if any.
    """
    speakers_string = ""
    function_content: Optional[str] = None
    if agent.data.speakers.max_retries_for_selecting is not None:
        speakers_string += (
            "    max_retries_for_selecting_speaker="
            f"{agent.data.speakers.max_retries_for_selecting},"
            "\n"
        )
    if agent.data.speakers.selection_method != "custom":
        speakers_string += (
            "    speaker_selection_method="
            f'"{agent.data.speakers.selection_method}",'
            "\n"
        )
    else:
        agent_name = agent_names[agent.id]
        function_content, function_name = (
            agent.data.speakers.get_custom_method_function(
                name_suffix=agent_name
            )
        )
        speakers_string += (
            f"    speaker_selection_method={function_name}," + "\n"
        )
    # selection_mode == "repeat":
    if agent.data.speakers.selection_mode == "repeat":
        speakers_string += _get_speakers_selection_repeat_string(
            agent, agent_names
        )
    # selection_mode == "transition":
    if (
        agent.data.speakers.selection_mode == "transition"
        and agent.data.speakers.allowed_or_disallowed_transitions
    ):
        speakers_string += _get_speakers_selection_transition_string(
            agent=agent,
            agent_names=agent_names,
            serializer=serializer,
        )
    speakers_string = speakers_string.replace('"None"', "None")
    return speakers_string, function_content


def _get_speakers_selection_repeat_string(
    agent: WaldiezGroupManager, agent_names: Dict[str, str]
) -> str:
    speakers_string = ""
    if isinstance(agent.data.speakers.allow_repeat, bool):
        speakers_string += (
            f"    allow_repeat_speaker={agent.data.speakers.allow_repeat},"
            + "\n"
        )
    elif isinstance(agent.data.speakers.allow_repeat, list):
        # get the names of the agents
        allow_repeat = ", ".join(
            agent_names[agent_id]
            for agent_id in agent.data.speakers.allow_repeat
        )
        speakers_string += f"    allow_repeat=[{allow_repeat}]," + "\n"
    return speakers_string


def _get_speakers_selection_transition_string(
    agent: WaldiezGroupManager,
    agent_names: Dict[str, str],
    serializer: Callable[..., str],
) -> str:
    speakers_string = ""
    allowed_or_disallowed_speaker_transitions = {}
    for (
        agent_id,
        transitions,
    ) in agent.data.speakers.allowed_or_disallowed_transitions.items():
        allowed_or_disallowed_speaker_transitions[agent_names[agent_id]] = [
            agent_names[transition] for transition in transitions
        ]
    transitions_string = serializer(
        allowed_or_disallowed_speaker_transitions, 1
    )
    transitions_string = transitions_string.replace('"', "").replace("'", "")
    speakers_string += (
        "    allowed_or_disallowed_speaker_transitions="
        f"{transitions_string}," + "\n"
    )
    speakers_string += (
        "    speaker_transitions_type="
        f'"{agent.data.speakers.transitions_type}",' + "\n"
    )
    return speakers_string



# SPDX-License-Identifier: Apache-2.0.
# Copyright (c) 2024 - 2025 Waldiez and contributors.
"""Skills/tools related string generation functions.

Functions
---------
get_agent_skill_registration
    Get an agent's skill registration string.
export_skills
    Get the skills content and secrets.
"""

from pathlib import Path
from typing import Callable, Dict, List, Optional, Tuple, Union

from waldiez.models import WaldiezAgent, WaldiezSkill


def get_agent_skill_registration(
    caller_name: str,
    executor_name: str,
    skill_name: str,
    skill_description: str,
    string_escape: Callable[[str], str],
) -> str:
    """Get the agent skill string and secrets.

    Parameters
    ----------
    caller_name : str
        The name of the caller (agent).
    executor_name : str
        The name of the executor (agent).
    skill_name : str
        The name of the skill.
    skill_description : str
        The skill description.
    string_escape : Callable[[str], str]
        The string escape function.
    Returns
    -------
    str
        The agent skill string.

    Example
    -------
    ```python
    >>> get_agent_skill_registration(
    ...     caller_name="agent1",
    ...     executor_name="agent2",
    ...     skill_name="skill1",
    ...     skill_description="A skill that does something.",
    ...     string_escape=lambda x: x.replace('"', '\\"').replace("\\n", "\\n"),
    ... )
    register_function(
        skill1,
        caller=agent1,
        executor=agent2,
        name="skill1",
        description="A skill that does something.",
    )
    ```
    """
    skill_description = string_escape(skill_description)
    content = f"""register_function(
    {skill_name},
    caller={caller_name},
    executor={executor_name},
    name="{skill_name}",
    description="{skill_description}",
)
"""
    return content


def _write_skill_secrets(
    flow_name: str,
    skill: WaldiezSkill,
    skill_name: str,
    output_dir: Optional[Union[str, Path]],
) -> None:
    """Write the skill secrets to a file.

    Parameters
    ----------
    skill : WaldiezSkill
        The skill.
    skill_name : str
        The name of the skill.
    output_dir : Optional[Union[str, Path]]
        The output directory to save the secrets to.
    """
    if not skill.secrets or not output_dir:
        return
    if not isinstance(output_dir, Path):
        output_dir = Path(output_dir)
    secrets_file = output_dir / f"{flow_name}_{skill_name}_secrets.py"
    first_line = f'"""Secrets for the skill: {skill_name}."""' + "\n"
    with secrets_file.open("w", encoding="utf-8", newline="\n") as f:
        f.write(first_line)
        f.write("import os\n\n")
        for key, value in skill.secrets.items():
            f.write(f'os.environ["{key}"] = "{value}"' + "\n")


def export_skills(
    flow_name: str,
    skills: List[WaldiezSkill],
    skill_names: Dict[str, str],
    output_dir: Optional[Union[str, Path]] = None,
) -> Tuple[List[str], List[Tuple[str, str]], str]:
    """Get the skills' contents and secrets.

    If `output_dir` is provided, the contents are saved to that directory.

    Parameters
    ----------
    flow_name : str
        The name of the flow.
    skills : List[WaldiezSkill]
        The skills.
    skill_names : Dict[str, str]
        The skill names.
    output_dir : Optional[Union[str, Path]]
        The output directory to save the skills to.

    Returns
    -------
    Tuple[Set[str], Set[Tuple[str, str]], str]
        - The skill imports to use in the main file.
        - The skill secrets to set as environment variables.
        - The skills contents.
    """
    skill_imports: List[str] = []
    skill_secrets: List[Tuple[str, str]] = []
    skill_contents: str = ""
    # if the skill.is_shared,
    # its contents must be first (before the other skills)
    shared_skill_contents = ""
    for skill in skills:
        skill_imports.append(get_skill_imports(flow_name, skill))
        for key, value in skill.secrets.items():
            skill_secrets.append((key, value))
        _write_skill_secrets(
            flow_name=flow_name,
            skill=skill,
            skill_name=skill_names[skill.id],
            output_dir=output_dir,
        )
        skill_content = skill.get_content()
        if not skill_content:
            continue
        if skill.is_shared:
            shared_skill_contents += skill_content + "\n\n"
        else:
            skill_contents += skill_content + "\n\n"
    skill_contents = shared_skill_contents + skill_contents
    return (
        skill_imports,
        skill_secrets,
        skill_contents.replace("\n\n\n", "\n\n"),
    )


def get_skill_imports(flow_name: str, skill: WaldiezSkill) -> str:
    """Get the skill imports string.

    Parameters
    ----------
    flow_name : str
        The name of the flow.
    skill : WaldiezSkill
        The skill.
    Returns
    -------
    str
        The skill imports string.
    """
    if not skill.secrets:
        return ""
        # fmt: on
    module_name = f"{flow_name}_{skill.name}"
    ignore_noqa = "  # type: ignore # noqa"
    return f"import {module_name}_secrets{ignore_noqa}" + "\n"


def get_agent_skill_registrations(
    agent: WaldiezAgent,
    agent_names: Dict[str, str],
    all_skills: List[WaldiezSkill],
    skill_names: Dict[str, str],
    string_escape: Callable[[str], str],
) -> str:
    """Get the agent skill registrations.

    example output:

    ```python
    >>> string_escape = lambda x: x.replace('"', '\\"').replace("\\n", "\\n")
    >>> agent = WaldiezAgent(
    ...     id="wa-1",
    ...     name="agent1",
    ...     description="An agent that does something.",
    ...     ...,
    ...     skills=[
    ...         WaldiezSkillLink(id="ws-1", executor_id="wa-2", ...),
    ...         WaldiezSkillLink(id="ws-2", executor_id="wa-3", ...),
    ...     ],
    ... )
    >>> agent_names = {"wa-1": "agent1", "wa-2": "agent2", "wa-3": "agent3"}
    >>> all_skills = [
    ...     WaldiezSkill(id="ws-1", ...),
    ...     WaldiezSkill(id="ws-2", ...),
    ...     WaldiezSkill(id="ws-3", ...),
    ... ]
    >>> skill_names = {"ws-1": "skill1", "ws-2": "skill2", "ws-3": "skill3"}
    >>> get_agent_skill_registrations(
    ...     agent=agent,
    ...     agent_names=agent_names,
    ...     all_skills=all_skills,
    ...     skill_names=skill_names,
    ...     string_escape=string_escape,
    ... )

    register_function(
        skill1,
        caller=agent1,
        executor=agent2,
        name="skill1",
        description="A skill that does something.",
    )
    register_function(
        skill2,
        caller=agent1,
        executor=agent3,
        name="skill2",
        description="A skill that does something.",
    )
    ```

    Parameters
    ----------
    agent : WaldiezAgent
        The agent.
    agent_names : Dict[str, str]
        A mapping of agent id to agent name.
    all_skills : List[WaldiezSkill]
        All the skills in the flow.
    skill_names : Dict[str, str]
        A mapping of skill id to skill name.
    string_escape : Callable[[str], str]
        The string escape function.
    Returns
    -------
    str
        The agent skill registrations.
    """
    if not agent.data.skills or not all_skills:
        return ""
    content = "\n"
    for linked_skill in agent.data.skills:
        waldiez_skill = next(
            skill for skill in all_skills if skill.id == linked_skill.id
        )
        skill_name = skill_names[linked_skill.id]
        skill_description = (
            waldiez_skill.description or f"Description of {skill_name}"
        )
        caller_name = agent_names[agent.id]
        executor_name = agent_names[linked_skill.executor_id]
        content += (
            get_agent_skill_registration(
                caller_name=caller_name,
                executor_name=executor_name,
                skill_name=skill_name,
                skill_description=skill_description,
                string_escape=string_escape,
            )
            + "\n"
        )
    return content.replace("\n\n\n", "\n\n")



# SPDX-License-Identifier: Apache-2.0.
# Copyright (c) 2024 - 2025 Waldiez and contributors.
"""Skills/tools related string generation functions.

Functions
---------
get_agent_skill_registration
    Get an agent's skill registration string.
export_skills
    Get the skills content and secrets.
"""

from pathlib import Path
from typing import Dict, List, Optional, Tuple, Union

from waldiez.models import WaldiezAgent, WaldiezSkill

from ..base import (
    AgentPosition,
    AgentPositions,
    BaseExporter,
    ExporterMixin,
    ExporterReturnType,
    ExportPosition,
    ImportPosition,
)
from .utils import export_skills, get_agent_skill_registrations


class SkillsExporter(BaseExporter, ExporterMixin):
    """Skill exporter."""

    def __init__(
        self,
        flow_name: str,
        agents: List[WaldiezAgent],
        agent_names: Dict[str, str],
        skills: List[WaldiezSkill],
        skill_names: Dict[str, str],
        output_dir: Optional[Union[str, Path]] = None,
    ) -> None:
        """Initialize the skill exporter.

        Parameters
        ----------
        flow_name : str
            The name of the flow.
        agents : List[WaldiezAgent]
            The agents.
        agent_names : Dict[str, str]
            The agent names.
        skills : List[WaldiezSkill]
            The skills.
        skill_names : Dict[str, str]
            The skill names.
        output_dir : Optional[Union[str, Path]], optional
            The output directory if any, by default None
        """
        self.flow_name = flow_name
        self.agents = agents
        self.agent_names = agent_names
        self.skills = skills
        self.skill_names = skill_names
        self.output_dir = output_dir
        self.skill_imports, self.skill_secrets, self.skills_contents = (
            export_skills(
                flow_name=flow_name,
                skills=skills,
                skill_names=skill_names,
                output_dir=output_dir,
            )
        )

    def get_environment_variables(self) -> List[Tuple[str, str]]:
        """Get the environment variables to set.

        Returns
        -------
        List[Tuple[str, str]]
            The environment variables to set.
        """
        return self.skill_secrets

    def get_imports(self) -> List[Tuple[str, ImportPosition]]:
        """ "Generate the imports string.

        Returns
        -------
        Tuple[str, int]
            The exported imports and the position of the imports.
        """
        if not self.skill_imports:
            return []
        imports: List[Tuple[str, ImportPosition]] = []
        for skill_import in self.skill_imports:
            if (skill_import, ImportPosition.LOCAL) not in imports:
                imports.append((skill_import, ImportPosition.LOCAL))
        return imports

    def get_before_export(
        self,
    ) -> Optional[List[Tuple[str, Union[ExportPosition, AgentPosition]]]]:
        """Generate the content before the main export.

        Returns
        -------
        Optional[List[Tuple[str, Union[ExportPosition, AgentPosition]]]]
            The exported content before the main export and its position.
        """

    def generate(self) -> Optional[str]:
        """Generate the main export.

        Returns
        -------
        Optional[str]
            The exported content.
        """
        return self.skills_contents

    def get_after_export(
        self,
    ) -> Optional[List[Tuple[str, Union[ExportPosition, AgentPosition]]]]:
        """Generate the content after the main export.

        Returns
        -------
        Optional[List[Tuple[str, Union[ExportPosition, AgentPosition]]]]
            The exported content after the main export and its position.
        """
        agent_registrations: List[
            Tuple[str, Union[ExportPosition, AgentPosition]]
        ] = []
        for agent in self.agents:
            agent_registration = get_agent_skill_registrations(
                agent=agent,
                agent_names=self.agent_names,
                all_skills=self.skills,
                skill_names=self.skill_names,
                string_escape=self.string_escape,
            )
            if agent_registration:
                # after all agents since we use the executor
                # (it might not yet be defined)
                position = AgentPosition(None, AgentPositions.AFTER_ALL, 1)
                agent_registrations.append((agent_registration, position))
        return agent_registrations

    def export(self) -> ExporterReturnType:
        """Export the skills.

        Returns
        -------
        ExporterReturnType
            The exported skills content, the imports,
            the before export strings, the after export strings,
            and the environment variables.
        """
        imports = self.get_imports()
        after_export = self.get_after_export()
        environment_variables = self.get_environment_variables()
        result: ExporterReturnType = {
            "content": self.generate(),
            "imports": imports,
            "before_export": None,
            "after_export": after_export,
            "environment_variables": environment_variables,
        }
        return result



# SPDX-License-Identifier: Apache-2.0.
# Copyright (c) 2024 - 2025 Waldiez and contributors.
"""Utilities for comments.
---------
comment
    Get a comment string.
get_comment
    Get a comment string for some common keys (notebook headings).
"""

from typing_extensions import Literal

CommentKey = Literal[
    "agents", "imports", "skills", "models", "nested", "run", "logging"
]


def comment(for_notebook: bool, hashtags: int = 1) -> str:
    """Get the comment string.

    Parameters
    ----------
    for_notebook : bool
        Whether the comment is for a notebook or not.
    hashtags : int, optional
        The number of hashtags (for notebooks), by default 1.

    Returns
    -------
    str
        The comment string.
    Example
    -------
    ```python
    >>> comment(True, 2)
    '## '
    >>> comment(False)
    '# '
    ```
    """
    content = "# "
    if for_notebook:
        content += "#" * hashtags + " "
    return content


def get_comment(
    key: CommentKey,
    for_notebook: bool,
) -> str:
    """Get a comment string for some common keys.

    The key is a heading (in a notebook) or just a comment (in a script).

    Parameters
    ----------
    key : "agents"|"imports"|"skills"|"models"|"nested"|"run"|"logging"
        The key.
    for_notebook : bool
        Whether the comment is for a notebook.

    Returns
    -------
    str
        The comment string.

    Example
    -------
    ```python
    >>> get_comment("agents", True)

    '## Agents'
    >>> get_comment("skills", False)

    '# Skills'
    ```
    """
    # pylint: disable=too-many-return-statements
    if key == "agents":
        return "\n" + comment(for_notebook, 2) + "Agents\n"
    if key == "imports":
        return "\n" + comment(for_notebook, 2) + "Imports\n"
    if key == "skills":
        return "\n" + comment(for_notebook, 2) + "Skills\n"
    if key == "models":
        return "\n" + comment(for_notebook, 2) + "Models\n"
    if key == "nested":
        return "\n" + comment(for_notebook, 2) + "Nested Chats\n"
    if key == "run":
        return "\n" + comment(for_notebook, 2) + "Run the flow\n"
    if key == "logging":
        return "\n" + comment(for_notebook, 2) + "Start Logging\n"
    return comment(for_notebook)
